import java.io.*;
import java.net.*;
import java.util.*;
import java.util.regex.*;
import java.text.*;
import javax.xml.xpath.*;
import java.io.File;

public class CacheVideo {

	public static final String LOCALROOT = "./cached-video";
	private static String nsLocation = null;
	URL videourl = null;
	String cachefile = null;
    static String ETHERNET_INTF="eth3";
	
    public CacheVideo() {
    }

	public CacheVideo(String url) {
		// Parse the URL
		try {
			videourl = new URL(url);
		} catch (Exception e) {
			output ("Problem casting URL - " + url);
			output (e.toString());
		}

		cachefile = getFileForURL (videourl);
        downloadURL();
	}
	
	public void output (String s) {
		System.out.println ("MicroCDNCache: " + s);
	}

	public String getFileForURL (URL videourl) {
		String host = videourl.getHost();
		String path = videourl.getPath();
		path = path.replaceAll ("~", "");
	
		String cache = LOCALROOT + "/" + host + "/" + path;
		File cachefile = new File(cache);
		cachefile.getParentFile().mkdirs();

		return cache;
	}
	
	public void downloadURL () {
		File cache = new File(cachefile);
		FileOutputStream out = null;

		URL u = videourl;
		// Code to download binary from URL, adapted from:
		// http://java2s.com/Tutorial/Java/0320__Network/SavebinaryfilefromURL.htm		
		try {
			URLConnection uc = u.openConnection();
			int contentLength = uc.getContentLength();
        System.out.println("URL = " + videourl);
			if (contentLength == -1) {
			  throw new IOException("This is not a binary file.");
			}
			InputStream raw = uc.getInputStream();
			InputStream in = new BufferedInputStream(raw);
			byte[] data = new byte[contentLength];
			int bytesRead = 0;
			int offset = 0;
			while (offset < contentLength) {
			  bytesRead = in.read(data, offset, data.length - offset);
			  if (bytesRead == -1)
				break;
			  offset += bytesRead;
			}
			in.close();

			out = new FileOutputStream(cache);
			 
			if (offset != contentLength) {
				throw new IOException("Only read " + offset 
					+ " bytes; Expected " + contentLength + " bytes");
			}

			out.write(data);
		} catch (Exception e) {
			output ("Problem downloading URL");
			output (e.toString());
		} finally {
			if (out != null) {
				try {
					out.flush();
					out.close();
				} catch (Exception e) {
					output ("Error closing output stream.");
					output (e.toString());
				}
			}
		}
	}

    public String getLocalFile(URL u)
    {
        return ("./cached-video" + u.getPath().replaceAll ("~", ""));
    }

    public static String getHostName()
    {
        String s;
        int i = 0;
        Pattern p = Pattern.compile("^[0-9]+.[0-9]+.[0-9]+.[0-9]+$");
        Matcher m;

        try {
            NetworkInterface ni = NetworkInterface.getByName(ETHERNET_INTF);
            List<InterfaceAddress> iAddrs = ni.getInterfaceAddresses();
            while ((s = iAddrs.get(i).getAddress().toString().substring(1)) != null) {
                m = p.matcher(s);
                if (m.matches()) {
                    return (s);
                }
                i++;
            }
            return (iAddrs.get(1).getAddress().toString().substring(1));
        } catch (Exception e) {
        }
        return "";
    }

    public String getLocalURL(URL u)
    {
            return ("http://" + getHostName() + ":8080" + u.getPath().replaceAll ("~", ""));
    }
}
